@@ -3389,8 +3389,9 @@ AS 'MODULE_PATHNAME';
 --
 -- query functions
 --
-CREATE FUNCTION ag_catalog.cypher(graph_name name, query_string cstring,
-                       params agtype = NULL)
+CREATE FUNCTION ag_catalog.cypher(graph_name name = NULL,
+                                  query_string cstring = NULL,
+                                  params agtype = NULL)
 RETURNS SETOF record
 LANGUAGE c
 AS 'MODULE_PATHNAME';
@@ -4160,6 +4161,13 @@ STABLE
 PARALLEL SAFE
 AS 'MODULE_PATHNAME';
 
+CREATE FUNCTION ag_catalog.age_prepare_cypher(cstring, cstring)
+RETURNS boolean
+LANGUAGE c
+STABLE
+PARALLEL SAFE
+AS 'MODULE_PATHNAME';
+
 --
 -- End
 --
