@@ -19,27 +19,23 @@
 
 #include "postgres.h"
 
-#include "access/sysattr.h"
+#include "access/heapam.h"
 #include "access/htup_details.h"
 #include "access/multixact.h"
+#include "access/table.h"
 #include "access/xact.h"
-#include "storage/bufmgr.h"
 #include "executor/tuptable.h"
 #include "nodes/execnodes.h"
 #include "nodes/extensible.h"
 #include "nodes/nodes.h"
 #include "nodes/plannodes.h"
 #include "parser/parsetree.h"
-#include "parser/parse_relation.h"
-#include "rewrite/rewriteHandler.h"
+#include "storage/bufmgr.h"
 #include "utils/rel.h"
-#include "utils/tqual.h"
 
 #include "catalog/ag_label.h"
-#include "commands/label_commands.h"
 #include "executor/cypher_executor.h"
 #include "executor/cypher_utils.h"
-#include "parser/cypher_parse_node.h"
 #include "nodes/cypher_nodes.h"
 #include "utils/agtype.h"
 #include "utils/graphid.h"
@@ -99,7 +95,8 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
 
     // setup scan tuple slot and projection info
     ExecInitScanTupleSlot(estate, &node->ss,
-                          ExecGetResultType(node->ss.ps.lefttree));
+                          ExecGetResultType(node->ss.ps.lefttree),
+                          &TTSOpsHeapTuple);
 
     if (!CYPHER_CLAUSE_IS_TERMINAL(css->flags))
     {
@@ -284,9 +281,9 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
 {
     ResultRelInfo *saved_resultRelInfo;
     LockTupleMode lockmode;
-    HeapUpdateFailureData hufd;
-    HTSU_Result lock_result;
-    HTSU_Result delete_result;
+    TM_FailureData hufd;
+    TM_Result lock_result;
+    TM_Result delete_result;
     Buffer buffer;
 
     // Find the physical tuple, this variable is coming from
@@ -303,11 +300,11 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
      * It is possible the entity may have already been deleted. If the tuple
      * can be deleted, the lock result will be HeapTupleMayBeUpdated. If the
      * tuple was already deleted by this DELETE clause, the result would be
-     * HeapTupleSelfUpdated, if the result was deleted by a previous delete
-     * clause, the result will HeapTupleInvisible. Throw an error if any
+     * TM_SelfModified, if the result was deleted by a previous delete
+     * clause, the result will TM_Invisible. Throw an error if any
      * other result was returned.
      */
-    if (lock_result == HeapTupleMayBeUpdated)
+    if (lock_result == TM_Ok)
     {
         delete_result = heap_delete(resultRelInfo->ri_RelationDesc,
                                     &tuple->t_self, GetCurrentCommandId(true),
@@ -320,30 +317,32 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
          */
         switch (delete_result)
         {
-                case HeapTupleMayBeUpdated:
-                        break;
-                case HeapTupleSelfUpdated:
-                        ereport(ERROR,
-                                (errcode(ERRCODE_INTERNAL_ERROR),
-                                         errmsg("deleting the same entity more than once cannot happen")));
-                        /* ereport never gets here */
-                        break;
-                case HeapTupleUpdated:
-                        ereport(ERROR,
-                                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
-                                         errmsg("could not serialize access due to concurrent update")));
-                        /* ereport never gets here */
-                        break;
-                default:
-                        elog(ERROR, "Entity failed to be update");
-                        /* elog never gets here */
-                        break;
+        case TM_Ok:
+            break;
+        case TM_SelfModified:
+            ereport(
+                ERROR,
+                (errcode(ERRCODE_INTERNAL_ERROR),
+                 errmsg(
+                     "deleting the same entity more than once cannot happen")));
+            /* ereport never gets here */
+            break;
+        case TM_Updated:
+            ereport(
+                ERROR,
+                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
+                 errmsg("could not serialize access due to concurrent update")));
+            /* ereport never gets here */
+            break;
+        default:
+            elog(ERROR, "Entity failed to be update");
+            /* elog never gets here */
+            break;
         }
         /* increment the command counter */
         CommandCounterIncrement();
     }
-    else if (lock_result != HeapTupleInvisible &&
-             lock_result != HeapTupleSelfUpdated)
+    else if (lock_result != TM_Invisible && lock_result != TM_SelfModified)
     {
         ereport(ERROR,
                 (errcode(ERRCODE_INTERNAL_ERROR),
@@ -374,7 +373,7 @@ static void process_delete_list(CustomScanState *node)
         cypher_delete_item *item;
         agtype_value *original_entity_value, *id, *label;
         ScanKeyData scan_keys[1];
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         ResultRelInfo *resultRelInfo;
         HeapTuple heap_tuple;
         char *label_name;
@@ -424,8 +423,8 @@ static void process_delete_list(CustomScanState *node)
         /*
          * Setup the scan description, with the correct snapshot and scan keys.
          */
-        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
-                                   estate->es_snapshot, 1, scan_keys);
+        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
+                                    estate->es_snapshot, 1, scan_keys);
 
         /* Retrieve the tuple. */
         heap_tuple = heap_getnext(scan_desc, ForwardScanDirection);
@@ -437,7 +436,7 @@ static void process_delete_list(CustomScanState *node)
          */
         if (!HeapTupleIsValid(heap_tuple))
         {
-            heap_endscan(scan_desc);
+            table_endscan(scan_desc);
             destroy_entity_result_rel_info(resultRelInfo);
 
             continue;
@@ -459,7 +458,7 @@ static void process_delete_list(CustomScanState *node)
         delete_entity(estate, resultRelInfo, heap_tuple);
 
         /* Close the scan and the relation. */
-        heap_endscan(scan_desc);
+        table_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
 }
@@ -492,18 +491,19 @@ static void find_connected_edges(CustomScanState *node, char *graph_name,
     {
         char *label_name = lfirst(lc);
         ResultRelInfo *resultRelInfo;
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         HeapTuple tuple;
         TupleTableSlot *slot;
 
         resultRelInfo = create_entity_result_rel_info(estate,
                                                       graph_name, label_name);
 
-        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
-                                   estate->es_snapshot, 0, NULL);
+        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
+                                    estate->es_snapshot, 0, NULL);
 
-        slot = ExecInitExtraTupleSlot(estate,
-                    RelationGetDescr(resultRelInfo->ri_RelationDesc));
+        slot = ExecInitExtraTupleSlot(
+            estate, RelationGetDescr(resultRelInfo->ri_RelationDesc),
+            &TTSOpsHeapTuple);
 
         // scan the table
         while(true)
@@ -517,7 +517,7 @@ static void find_connected_edges(CustomScanState *node, char *graph_name,
             if (!HeapTupleIsValid(tuple))
                 break;
 
-            ExecStoreTuple(tuple, slot, InvalidBuffer, false);
+            ExecStoreHeapTuple(tuple, slot, false);
 
             startid = GRAPHID_GET_DATUM(slot_getattr(slot, Anum_ag_label_edge_table_start_id, &isNull));
             endid = GRAPHID_GET_DATUM(slot_getattr(slot, Anum_ag_label_edge_table_end_id, &isNull));
@@ -540,7 +540,7 @@ static void find_connected_edges(CustomScanState *node, char *graph_name,
             }
         }
 
-        heap_endscan(scan_desc);
+        table_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
 
