@@ -19,14 +19,23 @@
 
 #include "postgres.h"
 
+#include "access/heapam.h"
+#include "access/relscan.h"
+#include "access/skey.h"
+#include "access/table.h"
+#include "access/tableam.h"
 #include "catalog/namespace.h"
+#include "commands/label_commands.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
 #include "utils/rel.h"
 #include "utils/snapmgr.h"
 #include "commands/label_commands.h"
 
+#include "catalog/ag_graph.h"
+#include "catalog/ag_label.h"
 #include "utils/age_global_graph.h"
+#include "utils/age_graphid_ds.h"
 #include "utils/agtype.h"
 #include "catalog/ag_graph.h"
 #include "catalog/ag_label.h"
@@ -189,7 +198,7 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
     List *labels = NIL;
     ScanKeyData scan_keys[2];
     Relation ag_label;
-    HeapScanDesc scan_desc;
+    TableScanDesc scan_desc;
     HeapTuple tuple;
     TupleDesc tupdesc;
 
@@ -203,8 +212,8 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
                 F_CHAREQ, CharGetDatum(label_type));
 
     /* setup the table to be scanned, ag_label in this case */
-    ag_label = heap_open(ag_label_relation_id(), ShareLock);
-    scan_desc = heap_beginscan(ag_label, snapshot, 2, scan_keys);
+    ag_label = table_open(ag_label_relation_id(), ShareLock);
+    scan_desc = table_beginscan(ag_label, snapshot, 2, scan_keys);
 
     /* get the tupdesc - we don't need to release this one */
     tupdesc = RelationGetDescr(ag_label);
@@ -228,8 +237,8 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
     }
 
     /* close up scan */
-    heap_endscan(scan_desc);
-    heap_close(ag_label, ShareLock);
+    table_endscan(scan_desc);
+    table_close(ag_label, ShareLock);
 
     return labels;
 }
@@ -399,7 +408,7 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
     foreach (lc, vertex_label_names)
     {
         Relation graph_vertex_label;
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         HeapTuple tuple;
         char *vertex_label_name;
         Oid vertex_label_table_oid;
@@ -411,8 +420,8 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
         vertex_label_table_oid = get_relname_relid(vertex_label_name,
                                                    graph_namespace_oid);
         /* open the relation (table) and begin the scan */
-        graph_vertex_label = heap_open(vertex_label_table_oid, ShareLock);
-        scan_desc = heap_beginscan(graph_vertex_label, snapshot, 0, NULL);
+        graph_vertex_label = table_open(vertex_label_table_oid, ShareLock);
+        scan_desc = table_beginscan(graph_vertex_label, snapshot, 0, NULL);
         /* get the tupdesc - we don't need to release this one */
         tupdesc = RelationGetDescr(graph_vertex_label);
         /* bail if the number of columns differs */
@@ -452,8 +461,8 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
         }
 
         /* end the scan and close the relation */
-        heap_endscan(scan_desc);
-        heap_close(graph_vertex_label, ShareLock);
+        table_endscan(scan_desc);
+        table_close(graph_vertex_label, ShareLock);
     }
 }
 
@@ -498,7 +507,7 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
     foreach (lc, edge_label_names)
     {
         Relation graph_edge_label;
-        HeapScanDesc scan_desc;
+        TableScanDesc scan_desc;
         HeapTuple tuple;
         char *edge_label_name;
         Oid edge_label_table_oid;
@@ -510,8 +519,8 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
         edge_label_table_oid = get_relname_relid(edge_label_name,
                                                  graph_namespace_oid);
         /* open the relation (table) and begin the scan */
-        graph_edge_label = heap_open(edge_label_table_oid, ShareLock);
-        scan_desc = heap_beginscan(graph_edge_label, snapshot, 0, NULL);
+        graph_edge_label = table_open(edge_label_table_oid, ShareLock);
+        scan_desc = table_beginscan(graph_edge_label, snapshot, 0, NULL);
         /* get the tupdesc - we don't need to release this one */
         tupdesc = RelationGetDescr(graph_edge_label);
         /* bail if the number of columns differs */
@@ -573,8 +582,8 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
         }
 
         /* end the scan and close the relation */
-        heap_endscan(scan_desc);
-        heap_close(graph_edge_label, ShareLock);
+        table_endscan(scan_desc);
+        table_close(graph_edge_label, ShareLock);
     }
 }
 
